@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 ## BleachBit
-## Copyright (C) 2009 Andrew Ziem
+## Copyright (C) 2010 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -29,7 +29,7 @@ import sys
 import traceback
 
 
-APP_VERSION = "0.6.3"
+APP_VERSION = "0.7.3"
 APP_NAME = "BleachBit"
 APP_URL = "http://bleachbit.sourceforge.net"
 
@@ -51,29 +51,35 @@ if hasattr(sys, 'frozen'):
     # running frozen in py2exe
     bleachbit_exe_path = os.path.dirname(unicode(sys.executable, sys.getfilesystemencoding()))
 else:
+    # __file__ is absolute path to bleachbit/Common.py
     bleachbit_exe_path = os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))
 
-# Debian, Ubuntu
-license_filename = "/usr/share/common-licenses/GPL-3"
-if not os.path.exists(license_filename):
-    # openSUSE 1.11
-    license_filename = "/usr/share/doc/packages/bleachbit/COPYING"
-if not os.path.exists(license_filename):
-    # Mandriva
-    license_filename = "/usr/share/doc/bleachbit/COPYING"
-if not os.path.exists(license_filename):
-    # CentOS, Fedora, RHEL
-    license_filename = "/usr/share/doc/bleachbit-" + APP_VERSION + "/COPYING"
-if not os.path.exists(license_filename) and sys.platform == 'win32':
-    # Windows
-    license_filename = os.path.join(bleachbit_exe_path, 'COPYING')
+# license
+license_filename = None
+license_filenames = ('/usr/share/common-licenses/GPL-3', # Debian, Ubuntu
+    os.path.join(bleachbit_exe_path, 'COPYING'), # Microsoft Windows
+    '/usr/share/doc/bleachbit-' + APP_VERSION + '/COPYING', # CentOS, Fedora, RHEL
+    '/usr/share/doc/packages/bleachbit/COPYING', # OpenSUSE 11.1
+    '/usr/share/doc/bleachbit/COPYING', # Mandriva
+    '/usr/pkg/share/doc/bleachbit/COPYING' ) # NetBSD 5
+for lf in license_filenames:
+    if os.path.exists(lf):
+        license_filename = lf
+        break
+if None == license_filename:
+    print 'warning: cannot find GPLv3 license text file'
 
 # configuration
 options_dir = None
-if sys.platform == 'linux2':
-    options_dir = os.path.expanduser("~/.config/bleachbit/")
-if sys.platform == 'win32':
-    options_dir = os.path.expandvars("${APPDATA}\\BleachBit\\")
+if 'posix' == os.name:
+    options_dir = os.path.expanduser("~/.config/bleachbit")
+elif 'nt' == os.name:
+    if os.path.exists(os.path.join(bleachbit_exe_path, 'bleachbit.ini')):
+        # portable mode
+        options_dir = bleachbit_exe_path
+    else:
+        # installed mode
+        options_dir = os.path.expandvars("${APPDATA}\\BleachBit")
 options_file = os.path.join(options_dir, "bleachbit.ini")
 
 # personal cleaners
@@ -83,30 +89,48 @@ personal_cleaners_dir = os.path.join(options_dir, "cleaners")
 system_cleaners_dir = None
 if sys.platform == 'linux2':
     system_cleaners_dir = '/usr/share/bleachbit/cleaners'
-if sys.platform == 'win32':
+elif sys.platform == 'win32':
     system_cleaners_dir = os.path.join(bleachbit_exe_path, 'share\\cleaners\\')
+elif sys.platform[:6] == 'netbsd':
+    system_cleaners_dir = '/usr/pkg/share/bleachbit/cleaners'
+
+# local cleaners directory (for running from source tree)
+local_cleaners_dir =  os.path.normpath(os.path.join(bleachbit_exe_path, '../cleaners'))
 
 # application icon
-if os.path.exists("bleachbit.png"):
-    appicon_path = "bleachbit.png"
-    print "debug: appicon_path = '%s'" % (appicon_path, )
-else:
-    if sys.platform == 'linux2':
-        appicon_path = "/usr/share/pixmaps/bleachbit.png"
-    if sys.platform == 'win32':
-        appicon_path = os.path.join(bleachbit_exe_path, 'share\\bleachbit.png')
+__icons = ( '/usr/share/pixmaps/bleachbit.png', # Linux
+    os.path.join(bleachbit_exe_path, 'share\\bleachbit.png'), # Windows
+    '/usr/pkg/share/pixmaps/bleachbit.png', # NetBSD
+    os.path.normpath(os.path.join(bleachbit_exe_path, '../bleachbit.png'))) # local
+appicon_path = None
+for __icon in __icons:
+    if os.path.exists(__icon):
+        appicon_path = __icon
+print "debug: appicon_path = '%s'" % (appicon_path, )
+
 
 # locale directory
 if os.path.exists("./locale/"):
-    # local locale
+    # local locale (personal)
     locale_dir = os.path.abspath("./locale/")
     print "debug: locale_dir = '%s'" % (locale_dir, )
 else:
-    # installed locale
+    # system-wide installed locale
     if sys.platform == 'linux2':
         locale_dir = "/usr/share/locale/"
-    if sys.platform == 'win32':
+    elif sys.platform == 'win32':
         locale_dir = os.path.join(bleachbit_exe_path, 'share\\locale\\')
+    elif sys.platform[:6] == 'netbsd':
+        locale_dir = "/usr/pkg/share/locale/"
+
+
+# launcher
+launcher_path = '/usr/share/applications/bleachbit.desktop'
+if 'posix' == os.name:
+    autostart_path = os.path.expanduser('~/.config/autostart/bleachbit.desktop')
+if 'nt' == os.name:
+    from win32com.shell import shell, shellcon
+    autostart_path = os.path.join(shell.SHGetSpecialFolderPath(None, shellcon.CSIDL_STARTUP), 'bleachbit.lnk')
 
 
 ###
@@ -116,22 +140,23 @@ else:
 try:
     user_locale = locale.getdefaultlocale()[0]
 except:
-    traceback.print_exc()
+    print 'warning: error getting locale: %s' % str(sys.exc_info()[1])
     user_locale = None
 
 if None == user_locale:
-    user_locale = 'en_US'
+    user_locale = 'C'
     print "warning: No default locale found.  Assuming '%s'" % user_locale
 
 if 'win32' == sys.platform:
     os.environ['LANG'] = user_locale
 
 try:
-    gettext.bindtextdomain('bleachbit', locale_dir)
-    gettext.textdomain('bleachbit')
-    gettext.install('bleachbit', locale_dir, unicode=1)
+    t = gettext.translation('bleachbit', locale_dir)
+    _ = t.ugettext
 except:
-    print "Warning: gettext() failed so translations will be unvailable"
+    if 'C' != user_locale and 'en' != user_locale[0:2]:
+        traceback.print_exc()
+        print "warning: gettext() failed so translations will be unavailable"
     def _(msg):
         """Dummy replacement for gettext"""
         return msg
