@@ -2,8 +2,8 @@
 # vim: ts=4:sw=4:expandtab
 
 ## BleachBit
-## Copyright (C) 2009 Andrew Ziem
-## http://bleachbit.sourceforge.net
+## Copyright (C) 2010 Andrew Ziem
+## http://sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -19,10 +19,10 @@
 ## along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-from gettext import gettext as _
 import os
 import sys
 import threading
+import time
 import traceback
 import types
 import warnings
@@ -34,15 +34,19 @@ import gtk
 import gobject
 warnings.simplefilter('default')
 
-from Common import APP_NAME, APP_VERSION, APP_URL, appicon_path, \
+from Common import _, APP_NAME, APP_VERSION, APP_URL, appicon_path, \
     help_contents_url, license_filename, online_update_notification_enabled, \
     release_notes_url
-from CleanerBackend import backends
+from Cleaner import backends
 from GuiPreferences import PreferencesDialog
 from Options import options
-import CleanerBackend
+import Cleaner
 import FileUtilities
-import Update
+import GuiBasic
+
+if 'nt' == os.name:
+    import Windows
+
 
 
 def open_url(url):
@@ -67,36 +71,6 @@ def threaded(func):
 
 
 
-def delete_confirmation_dialog(parent, mention_preview):
-    """Return boolean whether OK to delete files."""
-    dialog = gtk.Dialog(title = _("Delete confirmation"), parent = parent, flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
-    dialog.set_default_size(300, -1)
-
-    hbox = gtk.HBox(homogeneous=False, spacing=10)
-    icon = gtk.Image()
-    icon.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
-    hbox.pack_start(icon, False)
-    if mention_preview:
-        question_text = _("Are you sure you want to permanently delete files according to the selected operations?  The actual files that will be deleted may have changed since you ran the preview.")
-    else:
-        question_text = _("Are you sure you want to permanently delete these files?")
-
-    question = gtk.Label(question_text)
-    question.set_line_wrap(True)
-    hbox.pack_start(question, False)
-    dialog.vbox.pack_start(hbox, False)
-    dialog.vbox.set_spacing(10)
-
-    dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-    dialog.add_button(gtk.STOCK_DELETE, gtk.RESPONSE_ACCEPT)
-    dialog.set_default_response(gtk.RESPONSE_CANCEL)
-
-    dialog.show_all()
-    ret = dialog.run()
-    dialog.destroy()
-    return ret == gtk.RESPONSE_ACCEPT
-
-
 class TreeInfoModel:
     """Model holds information to be displayed in the tree view"""
 
@@ -140,7 +114,7 @@ class TreeInfoModel:
                 print "info: automatically hiding cleaner '%s'" % (c_id)
                 continue
             parent = self.tree_store.append(None, (c_name, c_value, c_id))
-            for (o_id, o_name, o_value) in backends[key].get_options():
+            for (o_id, o_name) in backends[key].get_options():
                 o_value = options.get_tree(c_id, o_id)
                 self.tree_store.append(parent, (o_name, o_value, o_id))
         self.row_changed_handler_id = self.tree_store.connect("row-changed", \
@@ -207,11 +181,9 @@ class TreeDisplayModel:
             # when toggling an option, present any warnings
             warning = backends[cleaner_id].get_warning(option_id)
             if warning:
-                dialog = gtk.MessageDialog(parent_window, gtk.DIALOG_MODAL, \
-                    gtk.MESSAGE_WARNING, gtk.BUTTONS_OK_CANCEL, \
-                    warning)
-                resp = dialog.run()
-                dialog.destroy()
+                resp = GuiBasic.message_dialog(parent_window, \
+                    warning, \
+                    gtk.MESSAGE_WARNING, gtk.BUTTONS_OK_CANCEL)
                 if gtk.RESPONSE_OK != resp:
                     # user cancelled, so don't toggle option
                     return
@@ -261,6 +233,7 @@ class GUI:
         <menu action="Help">
             <menuitem action="HelpContents"/>
             <menuitem action="ReleaseNotes"/>
+            <menuitem action="SystemInformation"/>
             <menuitem action="About"/>
         </menu>
     </menubar>
@@ -268,6 +241,7 @@ class GUI:
 
 
     def append_text(self, text, tag = None, __iter = None):
+        """Add some text to the main log"""
         if not __iter:
             __iter = self.textbuffer.get_end_iter()
         if tag:
@@ -332,8 +306,9 @@ class GUI:
                 if None == iterc:
                     return None
                 while iterc:
-                    tup = (model[iterc][2], model[iterc][1])
-                    ret.append(tup)
+                    if model[iterc][1]:
+                        # option is enabled
+                        ret.append(model[iterc][2])
                     iterc = model.iter_next(iterc)
                 return ret
             __iter = model.iter_next(__iter)
@@ -350,7 +325,7 @@ class GUI:
     def run_operations(self, __widget):
         """Event when the 'delete' toolbar button is clicked."""
         # fixme: should present this dialog after finding operations
-        if not delete_confirmation_dialog(self.window, True):
+        if not GuiBasic.delete_confirmation_dialog(self.window, True):
             return
         self.preview_or_run_operations(True)
 
@@ -358,47 +333,113 @@ class GUI:
     def preview_or_run_operations(self, really_delete, operations = None):
         """Preview operations or run operations (delete files)"""
 
-        assert(type(really_delete) is bool)
+        assert(isinstance(really_delete, bool))
         import Worker
+        self.start_time = None
         if None == operations:
             operations = {}
             for operation in self.get_selected_operations():
                 operations[operation] = self.get_operation_options(operation)
-        assert(type(operations) is dict)
+        assert(isinstance(operations, dict))
+        if 0 == len(operations):
+            GuiBasic.message_dialog(self.window, \
+                _("You must select an operation"),
+                gtk.MESSAGE_WARNING, gtk.BUTTONS_OK)
+            return
         try:
+            self.set_sensitive(False)
+            self.textbuffer.set_text("")
+            self.progressbar.show()
             self.worker = Worker.Worker(self, really_delete, operations)
         except:
             traceback.print_exc()
             err = str(sys.exc_info()[1])
             self.append_text(err + "\n", 'error')
         else:
-            self.worker.set_total_size_cb(self.cb_total_size)
+            self.start_time = time.time()
             worker = self.worker.run()
             gobject.idle_add(worker.next)
 
 
+
+    def worker_done(self, worker, really_delete):
+        """Callback for when Worker is done"""
+        self.progressbar.set_text("")
+        self.progressbar.set_fraction(1)
+        self.progressbar.set_text(_("Done."))
+        self.textview.scroll_mark_onscreen(self.textbuffer.get_insert())
+        self.set_sensitive(True)
+
+        # notification for long-running process
+        elapsed = (time.time() - self.start_time)
+        print 'debug: elapsed time: %d seconds' % elapsed
+        if elapsed < 10 or self.window.is_active():
+            return
+        try:
+            import pynotify
+        except:
+            print "debug: pynotify not available"
+        else:
+            if pynotify.init(APP_NAME):
+                notify = pynotify.Notification('BleachBit', _("Done."), \
+                    icon = 'bleachbit')
+                notify.show()
+                notify.set_timeout(10000)
+
+
+
     def about(self, __event):
         """Create and show the about dialog"""
         gtk.about_dialog_set_url_hook(lambda dialog, link: open_url(link))
         dialog = gtk.AboutDialog()
         dialog.set_comments(_("Program to clean unnecessary files"))
-        dialog.set_copyright("Copyright (C) 2009 Andrew Ziem")
+        dialog.set_copyright("Copyright (C) 2010 Andrew Ziem")
         try:
             dialog.set_license(open(license_filename).read())
         except:
             dialog.set_license(_("GNU General Public License version 3 or later.\nSee http://www.gnu.org/licenses/gpl-3.0.txt"))
         dialog.set_name(APP_NAME)
-        # TRANSLASTORS: Maintain the names of translators here.
+        # TRANSLATORS: Maintain the names of translators here.
         # Launchpad does this automatically for translations
-        # typed in Launchpad. This is a special string shown 
+        # typed in Launchpad. This is a special string shown
         # in the 'About' box.
         dialog.set_translator_credits(_("translator-credits"))
         dialog.set_version(APP_VERSION)
         dialog.set_website(APP_URL)
+        dialog.set_transient_for(self.window)
+        if os.path.exists(appicon_path):
+            icon = gtk.gdk.pixbuf_new_from_file(appicon_path)
+            dialog.set_logo(icon)
         dialog.run()
         dialog.hide()
 
 
+    def diagnostic_dialog(self, parent):
+        """Show diagnostic information"""
+        dialog = gtk.Dialog(_("System information"), parent)
+        dialog.resize(600, 400)
+        buffer = gtk.TextBuffer()
+        import Diagnostic
+        txt = Diagnostic.diagnostic_info()
+        buffer.set_text(txt)
+        textview = gtk.TextView(buffer)
+        textview.set_editable(False)
+        swindow = gtk.ScrolledWindow()
+        swindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        swindow.add_with_viewport(textview)
+        dialog.vbox.pack_start(swindow)
+        dialog.add_buttons(gtk.STOCK_COPY, 100, gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+        dialog.show_all()
+        while True:
+            rc = dialog.run()
+            if 100 == rc:
+                clipboard = gtk.clipboard_get()
+                clipboard.set_text(txt)
+            else:
+                break
+        dialog.hide()
+
+
     def create_operations_box(self):
         """Create and return the operations box (which holds a tree view)"""
         scrolled_window = gtk.ScrolledWindow()
@@ -428,48 +469,41 @@ class GUI:
         """Callback for shredding a file"""
 
         # get list of files
-        chooser = gtk.FileChooserDialog(title = _("Choose files to shred"),
-            parent = self.window,
-            action = gtk.FILE_CHOOSER_ACTION_OPEN,
-            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_DELETE, gtk.RESPONSE_OK))
-        chooser.set_select_multiple(True)
-        resp = chooser.run()
-        paths = chooser.get_filenames()
-        chooser.destroy()
+        paths = GuiBasic.browse_files(self.window, \
+            _("Choose files to shred"))
 
-        if gtk.RESPONSE_OK != resp:
+        if not paths:
             return
 
         # create a temporary cleaner object
-        cleaner = CleanerBackend.Cleaner()
-        cleaner.add_option('files', 'files', '')
-        cleaner.name = ''
-        import Action
-        actioncontainer = Action.ActionContainer()
-        class CustomFileAction(Action.ActionProvider):
-            def list_files(self):
-                for path in paths:
-                    yield path
-        provider = CustomFileAction(None)
-        actioncontainer.add_action_provider(provider)
-        cleaner.add_action('files', actioncontainer)
-        backends['_gui'] = cleaner
+        backends['_gui'] = Cleaner.create_simple_cleaner(paths)
 
         # preview and confirm
-        operations = { '_gui' : [ ( 'files', True) ] }
+        operations = { '_gui' : [ 'files' ] }
         self.preview_or_run_operations(False, operations)
 
-        if delete_confirmation_dialog(self.window, mention_preview = False):
+        if GuiBasic.delete_confirmation_dialog(self.window, mention_preview = False):
             # delete
+            options.set('shred', True, commit = False)
             self.preview_or_run_operations(True, operations)
+            options.restore()
             return
 
         # clean up temporary cleaner
         del backends['_gui']
 
 
+    def update_progress_bar(self, status):
+        """Callback to update the progress bar with number or text"""
+        if type(status) is float:
+            self.progressbar.set_fraction(status)
+        elif (type(status) is str) or (type(status) is unicode):
+            self.progressbar.set_text(status)
+        else:
+            raise RuntimeError('unexpected type: ' + str(type(status)))
 
-    def cb_total_size(self, bytes):
+
+    def update_total_size(self, bytes):
         """Callback to update the total size cleaned"""
         context_id = self.status_bar.get_context_id('size')
         text = FileUtilities.bytes_to_human(bytes)
@@ -483,7 +517,7 @@ class GUI:
         # Create a UIManager instance
         uimanager = gtk.UIManager()
 
-        # Add the accelerator group to the toplevel window
+        # Add the accelerator group to the top level window
         accelgroup = uimanager.get_accel_group()
         self.window.add_accel_group(accelgroup)
 
@@ -500,6 +534,7 @@ class GUI:
                     ('Edit', None, _("_Edit")),
                     ('HelpContents', gtk.STOCK_HELP, _('Help Contents'), 'F1', None, lambda link: open_url(help_contents_url)),
                     ('ReleaseNotes', gtk.STOCK_INFO, _('_Release Notes'), None, None, lambda link: open_url(release_notes_url)),
+                    ('SystemInformation', None, _('_System Information'), None, None, lambda foo: self.diagnostic_dialog(self.window)),
                     ('About', gtk.STOCK_ABOUT, _('_About'), None, None, self.about),
                     ('Help', None, _("_Help")))
         actiongroup.add_actions(entries)
@@ -519,9 +554,6 @@ class GUI:
     def create_toolbar(self):
         """Create the toolbar"""
         toolbar = gtk.Toolbar()
-        toolbar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
-        toolbar.set_style(gtk.TOOLBAR_BOTH)
-        toolbar.set_border_width(5)
 
         # create tooltips
         tooltips = gtk.Tooltips()
@@ -534,26 +566,32 @@ class GUI:
         preview_button.connect("clicked", lambda *dummy: self.preview_or_run_operations(False))
         toolbar.insert(preview_button, -1)
         preview_button.set_tooltip(tooltips, _("Preview files in the selected operations (without deleting any files)"))
+        preview_button.set_is_important(True)
 
         # create the delete button
         icon = gtk.Image()
         icon.set_from_stock(gtk.STOCK_DELETE, gtk.ICON_SIZE_LARGE_TOOLBAR)
+        # TRANSLATORS: This is the delete button on the main window
         run_button = gtk.ToolButton(icon_widget = icon, label = _("Delete"))
         run_button.connect("clicked", self.run_operations)
         toolbar.insert(run_button, -1)
         run_button.set_tooltip(tooltips, _("Delete files in the selected operations"))
+        run_button.set_is_important(True)
 
         return toolbar
 
 
     def create_window(self):
         """Create the main application window"""
+
+        print 'info: GTK version %s' % '.'.join([str(x) for x in gtk.gtk_version])
+
         self.window = gtk.Window()
         self.window.connect('destroy', lambda w: gtk.main_quit())
 
         self.window.resize(800, 600)
         self.window.set_title(APP_NAME)
-        if os.path.exists(appicon_path):
+        if appicon_path and os.path.exists(appicon_path):
             self.window.set_icon_from_file(appicon_path)
         vbox = gtk.VBox()
         self.window.add(vbox)
@@ -621,14 +659,17 @@ class GUI:
         update_button = gtk.ToolButton(icon_widget = icon, label = _("Update BleachBit"))
         update_button.show_all()
         update_button.connect("clicked", lambda toolbutton, url: open_url(url), url)
+        update_button.set_is_important(True)
         self.toolbar.insert(update_button, -1)
+        msg = _("BleachBit update is available")
         try:
             import pynotify
         except:
             print "debug: pynotify not available"
+            self.append_text(msg)
         else:
             if pynotify.init(APP_NAME):
-                notify = pynotify.Notification(_("BleachBit update is available"), _("Click 'Update BleachBit' for more information"))
+                notify = pynotify.Notification(msg, _("Click 'Update BleachBit' for more information"))
                 # this doesn't align the notification properly
                 #n.attach_to_widget(update_button)
                 notify.attach_to_widget(self.toolbar)
@@ -638,19 +679,26 @@ class GUI:
     @threaded
     def check_online_updates(self):
         """Check for software updates in background"""
-        update = Update.Update()
-        if update.is_update_available():
-            gobject.idle_add(self.enable_online_update, update.get_update_info_url())
+        import Update
+        try:
+            update = Update.Update()
+            if update.is_update_available():
+                gobject.idle_add(self.enable_online_update, update.get_update_info_url())
+        except:
+            self.append_text(_("Error when checking for updates: ") + str(sys.exc_info()[1]))
 
 
     def __init__(self):
+        if 'nt' == os.name and Windows.elevate_privileges():
+            # privileges escalated in other process
+            sys.exit(0)
         import RecognizeCleanerML
         RecognizeCleanerML.RecognizeCleanerML()
         import CleanerML
         CleanerML.load_cleaners()
         self.create_window()
         gobject.threads_init()
-        if options.get("first_start") and sys.platform == 'linux2':
+        if options.get("first_start") and 'posix' == os.name:
             pref = PreferencesDialog(self.window, self.cb_refresh_operations)
             pref.run()
             options.set('first_start', False)
@@ -658,6 +706,7 @@ class GUI:
             self.check_online_updates()
 
 
+
 if __name__ == '__main__':
     gui = GUI()
     gtk.main()
