@@ -1,8 +1,9 @@
-#!/usr/bin/env python
 # vim: ts=4:sw=4:expandtab
+# -*- coding: UTF-8 -*-
+
 
 ## BleachBit
-## Copyright (C) 2009 Andrew Ziem
+## Copyright (C) 2010 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -18,16 +19,27 @@
 ## You should have received a copy of the GNU General Public License
 ## along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+
+
 """
 Preferences dialog
 """
 
 
 import gtk
+import os
 import sys
+import traceback
 
-from Common import online_update_notification_enabled
+from Common import _
 from Options import options
+import Common
+import GuiBasic
+
+if 'nt' == os.name:
+    import Windows
+if 'posix' == os.name:
+    import Unix
 
 
 
@@ -49,28 +61,52 @@ class PreferencesDialog:
         notebook = gtk.Notebook()
         notebook.append_page(self.__general_page(), gtk.Label(_("General")))
         notebook.append_page(self.__drives_page(), gtk.Label(_("Drives")))
-        if sys.platform == 'linux2':
+        if 'posix' == os.name:
             notebook.append_page(self.__languages_page(), gtk.Label(_("Languages")))
 
-        self.dialog.vbox.pack_start(notebook, False)
+        self.dialog.vbox.pack_start(notebook, True)
         self.dialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
 
 
+    def __toggle_callback(self, cell, path):
+        """Callback function to toggle option"""
+        options.toggle(path)
+        if 'auto_hide' == path:
+            self.cb_refresh_operations()
+        if 'auto_start' == path:
+            if 'nt' == os.name:
+                swc = Windows.start_with_computer
+            if 'posix' == os.name:
+                swc = Unix.start_with_computer
+            try:
+                swc(options.get(path))
+            except:
+                traceback.print_exc()
+                dlg = gtk.MessageDialog(self.parent,
+                    type = gtk.MESSAGE_ERROR,
+                    buttons = gtk.BUTTONS_OK,
+                    message_format = str(sys.exc_info()[1]))
+                dlg.run()
+                dlg.destroy()
+
+
     def __general_page(self):
         """Return a widget containing the general page"""
 
-        def toggle_callback(cell, path):
-            """Callback function to toggle option b"""
-            options.toggle(path)
-            if 'auto_hide' == path:
-                self.cb_refresh_operations()
+
+        if 'nt' == os.name:
+            swcc = Windows.start_with_computer_check
+        if 'posix' == os.name:
+            swcc = Unix.start_with_computer_check
+
+        options.set('auto_start', swcc())
 
         vbox = gtk.VBox()
 
-        if online_update_notification_enabled:
+        if Common.online_update_notification_enabled:
             cb_updates = gtk.CheckButton(_("Check periodically for software updates via the Internet"))
             cb_updates.set_active(options.get('check_online_updates'))
-            cb_updates.connect('toggled', toggle_callback, 'check_online_updates')
+            cb_updates.connect('toggled', self.__toggle_callback, 'check_online_updates')
             self.tooltips.set_tip(cb_updates, _("If an update is found, you will be given the option to view information about it.  Then, you may manually download and install the update."))
             vbox.pack_start(cb_updates, False)
 
@@ -80,7 +116,7 @@ class PreferencesDialog:
         # the list of cleaners.
         cb_auto_hide = gtk.CheckButton(_("Hide irrelevant cleaners"))
         cb_auto_hide.set_active(options.get('auto_hide'))
-        cb_auto_hide.connect('toggled', toggle_callback, 'auto_hide')
+        cb_auto_hide.connect('toggled', self.__toggle_callback, 'auto_hide')
         vbox.pack_start(cb_auto_hide, False)
 
         # TRANSLATORS: Overwriting is the same as shredding.  It is a way
@@ -88,11 +124,14 @@ class PreferencesDialog:
         # 'Shred files to prevent recovery.'
         cb_shred = gtk.CheckButton(_("Overwrite files to hide contents"))
         cb_shred.set_active(options.get('shred'))
-        cb_shred.connect('toggled', toggle_callback, 'shred')
+        cb_shred.connect('toggled', self.__toggle_callback, 'shred')
         self.tooltips.set_tip(cb_shred, _("Overwriting is ineffective on some file systems and with certain BleachBit operations.  Overwriting is significantly slower."))
         vbox.pack_start(cb_shred, False)
 
-
+        cb_start = gtk.CheckButton(_("Start BleachBit with computer"))
+        cb_start.set_active(options.get('auto_start'))
+        cb_start.connect('toggled', self.__toggle_callback, 'auto_start')
+        vbox.pack_start(cb_start, False)
         return vbox
 
     def __drives_page(self):
@@ -100,17 +139,9 @@ class PreferencesDialog:
 
         def add_drive_cb(button):
             """Callback for adding a drive"""
-            chooser = gtk.FileChooserDialog( parent = self.parent, \
-                title = _("Choose a folder"),
-                buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, \
-                gtk.STOCK_ADD, gtk.RESPONSE_ACCEPT), \
-                action = gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
-
-            resp = chooser.run()
-            pathname = chooser.get_filename()
-            chooser.hide()
-            chooser.destroy()
-            if gtk.RESPONSE_ACCEPT == resp:
+            title = _("Choose a folder")
+            pathname = GuiBasic.browse_folder(self.parent, title)
+            if pathname:
                 liststore.append([pathname])
                 pathnames.append(pathname)
                 options.set_list('shred_drives', pathnames)
@@ -128,7 +159,8 @@ class PreferencesDialog:
 
         vbox = gtk.VBox()
 
-        notice = gtk.Label(_("Choose a writeable directory for each drive for which to overwrite free space."))
+        notice = gtk.Label(_("Choose a writable directory for each drive for which to overwrite free space."))
+        notice.set_line_wrap(True)
         vbox.pack_start(notice, False)
 
         liststore = gtk.ListStore(str)
