@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 ## BleachBit
-## Copyright (C) 2009 Andrew Ziem
+## Copyright (C) 2010 Andrew Ziem
 ## http://bleachbit.sourceforge.net
 ##
 ## This program is free software: you can redistribute it and/or modify
@@ -19,6 +19,7 @@
 ## along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
+
 """
 Integration specific to Unix-like operating systems
 """
@@ -32,6 +33,8 @@ import subprocess
 import sys
 import ConfigParser
 
+from Common import _
+import Common
 import FileUtilities
 import General
 
@@ -110,12 +113,14 @@ class Locales:
         'hi' : 'हिन्दी', \
         'hr' : 'Hrvatski', \
         'hu' : 'Magyar', \
+        'id' : 'Indonesian', \
         'it' : 'Italiano', \
         'iw' : 'עברית', \
         'ja' : '日本語', \
         'jv' : 'basa Jawa', \
         'ko' : '한국어', \
         'mr' : 'मराठी', \
+        'ms' : 'بهاس ملايو', \
         'nl' : 'Nederlands', \
         'no' : 'Norsk', \
         'pl' : 'polski', \
@@ -527,6 +532,28 @@ def rotated_logs():
             yield path
 
 
+def start_with_computer(enabled):
+    """If enabled, create shortcut to start application with computer.
+    If disabled, then delete the shortcut."""
+    if not enabled:
+        if os.path.lexists(Common.autostart_path):
+            FileUtilities.delete(Common.autostart_path)
+        return
+    if os.path.lexists(Common.autostart_path):
+        return
+    import shutil
+    General.makedirs(os.path.dirname(Common.autostart_path))
+    shutil.copy(Common.launcher_path, Common.autostart_path)
+    os.chmod(Common.autostart_path, 0755)
+    if General.sudo_mode():
+        General.chownself(Common.autostart_path)
+
+
+def start_with_computer_check():
+    """Return boolean whether BleachBit will start with the computer"""
+    return os.path.lexists(Common.autostart_path)
+
+
 def wine_to_linux_path(wineprefix, windows_pathname):
     """Return a Linux pathname from an absolute Windows pathname and Wine prefix"""
     drive_letter = windows_pathname[0]
@@ -547,8 +574,11 @@ def yum_clean():
     args  = ['yum', "--enablerepo=*", 'clean', 'all']
     p = subprocess.Popen(args, stderr=subprocess.STDOUT, \
         stdout=subprocess.PIPE)
+    non_blank_line = ""
     while True:
         line = p.stdout.readline().replace("\n", "")
+        if len(line) > 2:
+            non_blank_line = line
         if -1 != line.find('You need to be root'):
             raise RuntimeError(line)
         if -1 != line.find('Another app is currently holding'):
@@ -557,6 +587,8 @@ def yum_clean():
         if "" == line and p.poll() != None:
             break
     print 'debug: yum process return code = %d' % p.returncode
+    if p.returncode > 0:
+        raise RuntimeError(non_blank_line)
     new_size = FileUtilities.getsizedir('/var/cache/yum')
     return old_size - new_size
 
@@ -564,146 +596,4 @@ def yum_clean():
 locales = Locales()
 
 
-import unittest
-
-class TestUnix(unittest.TestCase):
-    """Unit tests for module Unix"""
-
-    def setUp(self):
-        """Initialize unit tests"""
-        self.locales = Locales()
-
-
-    def test_apt_autoclean(self):
-        """Unit test for method apt_autoclean()"""
-        if 0 != os.geteuid() or not FileUtilities.exe_exists('apt-get'):
-            self.assertRaises(RuntimeError, apt_autoclean)
-        else:
-            bytes_freed = apt_autoclean()
-            self.assert_(type(bytes_freed) is int)
-
-
-    def test_free_space(self):
-        """Unit test for free_space()"""
-        home = os.path.expanduser("~")
-        result = free_space(home)
-        self.assertNotEqual(result, None)
-        self.assert_(result > -1)
-        self.assert_(type(result) is int)
-
-
-    def test_guess_overwrite_paths(self):
-        """Unit test for guess_overwrite_paths()"""
-        for path in guess_overwrite_paths():
-            self.assert_(os.path.isdir(path))
-
-
-    def test_is_broken_xdg_desktop(self):
-        """Unit test for is_broken_xdg_desktop()"""
-        menu_dirs = [ '/usr/share/applications', \
-            '/usr/share/autostart', \
-            '/usr/share/gnome/autostart', \
-            '/usr/share/gnome/apps', \
-            '/usr/share/mimelnk', \
-            '/usr/share/applnk-redhat/', \
-            '/usr/local/share/applications/' ]
-        for dirname in menu_dirs:
-            for filename in [fn for fn in FileUtilities.children_in_directory(dirname, False) \
-                if fn.endswith('.desktop')]:
-                self.assert_(type(is_broken_xdg_desktop(filename) is bool))
-
-
-    def test_is_running(self):
-        exe = os.path.basename(sys.executable)
-        self.assertEqual(True, is_running(exe))
-
-
-    def test_iterate_languages(self):
-        """Unit test for the method iterate_languages()"""
-        for language in self.locales.iterate_languages():
-            self.assert_(type(language) is str)
-            self.assert_(len(language) > 0)
-            self.assert_(len(language) < 9)
-
-
-    def test_locale_to_language(self):
-        """Unit test for locale_to_language()"""
-        tests = [ ('en', 'en'),
-            ('en_US', 'en'),
-            ('en_US@piglatin', 'en'),
-            ('en_US.utf8', 'en'),
-            ('klingon', 'klingon'),
-            ('pl.ISO8859-2', 'pl'),
-            ('sr_Latn', 'sr'),
-            ('zh_TW.Big5', 'zh') ]
-        for test in tests:
-            self.assertEqual(locale_to_language(test[0]), test[1])
-
-        self.assertRaises(ValueError, locale_to_language, 'default')
-        self.assertRaises(ValueError, locale_to_language, 'C')
-        self.assertRaises(ValueError, locale_to_language, 'English')
-
-    def test_locale_globex(self):
-        """Unit test for locale_globex"""
-
-        locale = locale_globex('/bin/ls', '(ls)$').next()
-        self.assertEqual(locale, ('ls', '/bin/ls'))
-
-        fakepath = '/usr/share/omf/gedit/gedit-es.omf'
-
-        def test_yield(pathname, regex):
-            """Replacement for globex()"""
-            yield fakepath
-
-        old_globex = FileUtilities.globex
-        FileUtilities.globex = test_yield
-
-        func = locale_globex('/usr/share/omf/*/*-*.omf', '-([a-z]{2}).omf$')
-        actual = func.next()
-        expect = ('es', fakepath)
-        self.assertEqual(actual, expect, "Expected '%s' but got '%s'" % (expect, actual))
-        FileUtilities.globex = old_globex
-
-    def test_localization_paths(self):
-        """Unit test for localization_paths()"""
-        for path in locales.localization_paths(lambda x, y: False):
-            self.assert_(os.path.lexists(path))
-
-
-    def test_native_name(self):
-        """Unit test for native_name()"""
-        tests = [ ('en', 'English'),
-            ('es', 'Español') ]
-        for test in tests:
-            self.assertEqual(self.locales.native_name(test[0]), test[1])
-
-
-    def test_rotated_logs(self):
-        """Unit test for rotated_logs()"""
-        for path in rotated_logs():
-            self.assert_(os.path.exists(path), \
-                "Rotated log path '%s' does not exist" % path)
-
-
-    def test_wine_to_linux_path(self):
-        """Unit test for wine_to_linux_path()"""
-        tests = [ ("/home/foo/.wine", \
-            "C:\\Program Files\\NSIS\\NSIS.exe", \
-            "/home/foo/.wine/drive_c/Program Files/NSIS/NSIS.exe") ]
-        for test in tests:
-            self.assertEqual(wine_to_linux_path(test[0], test[1]), test[2])
-
-    def test_yum_clean(self):
-        """Unit test for yum_clean()"""
-        if 0 != os.geteuid() or os.path.exists('/var/run/yum.pid') \
-            or not FileUtilities.exe_exists('yum'):
-            self.assertRaises(RuntimeError, yum_clean)
-        else:
-            bytes_freed = yum_clean()
-            self.assert_(type(bytes_freed) is int)
-            print 'debug: yum bytes cleaned %d', bytes_freed
-
-
-if __name__ == '__main__' and 'linux2' == sys.platform:
-    unittest.main()
 
